from ucloudbiz.server.lib.api import UcloudAPI

class listPortForwardingRules(UcloudAPI):

    params = {'account'  : [False,None],
              'domainid' : [False,None],
              'id'       : [False,None],
              'ipaddressid'            : [False,None],
              'keyward'                : [False,None],
              'page'      	       : [False,None],
              'pagesize'               : [False,None],
              'isrecursive'            : [False,None],
              'listall'  	       : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




import urllib
import urllib2
from hashlib import sha1
import hmac
import binascii

class UcloudAPI:
    URL=None
    APIKEY=None
    SECRET=None
    RESPONSE='json'
    # format 
    #  key : [required(True/False), value]
    params = {}
    
    def __init__(self, url=None, apikey=None, secret=None):
        self.URL = url
        self.APIKEY = apikey
        self.SECRET = secret
        
    def helpParams(self):
        keys = self.params.keys()
        print "Parameter List"
        for key in keys:
            [tf, value] = self.params[key]
            if tf == True:
                print "%s\trequired" % key
            else:
                print "%s\toptional" % key

    def showParams(self):
        keys = self.params.keys()
        print "Parameter List"
        for key in keys:
            [tf, value] = self.params[key]
            if tf == True:
                print "%s\t= %s\t(required)" % (key, value)
            else:
                print "%s\t = %s\t(optional)" % (key, value)
                                                                                
                
    def checkParams(self):
        keys = self.params.keys()
        for key in keys:
            [tf, value] = self.params[key]
            if tf == True:
                if value == None:
                    print "Error: param(%s=%s) must exist" % (key, value)

    def updateParams(self, dic):
        """
        update self.params based on dic
        """
        keys = self.params.keys()
        for key in keys:
            if dic.has_key(key) == True:
                (tf, value) = self.params[key]
                self.params[key] = (tf, dic[key])
            #clean up, garbage if possible
            else:
                (tf, value) = self.params[key]
                self.params[key] = (tf, None)
        #self.showParams()
        
        
    def setURL(self, url):
        self.URL = url

    def setAPIKEY(self, apikey):
        self.APIKEY = apikey

    def setSECRET(self, secret):
        self.SECRET = secret


    def createAPI(self):
    	#
	real_params = {'command':self.__class__.__name__}
	real_params['apikey'] = self.APIKEY
	real_params['signature'] = self.createSignature()
	real_params['response'] = self.RESPONSE
	keys = self.params.keys()
	for key in keys:
	    test = self.params[key]
	    if test[1] == None:
	       continue
	    real_params[key] = test[1]

        url_params = urllib.urlencode(real_params).replace('+','%20')	
	return "%s%s" % (self.URL, url_params)
	 
    def createSignature(self):
        # filter real parameters
        keys = self.params.keys()
        # add command name
        real_params = {'command':(self.__class__.__name__).lower()}
        for key in keys:
	    test = self.params[key]
            if test[1] == None:
                continue
	    low_str = test[1]
	    try:
		toLower = low_str.lower()
	    except:
		#This seems like Number
		toLower = low_str
            real_params[key.lower()] = toLower
            
        # add response type
        real_params['response'] = (self.RESPONSE).lower()
        # add apikey
	low_str = self.APIKEY
        real_params['apikey'] = low_str.lower()
         
        url_params = urllib.urlencode(real_params).replace('+','%20')

	#print url_params
	items = url_params.split("&")
	url_params_new = {}
	for item in items:
	    index = item.find("=")
	    url_params_new[item[:index]]=item[(index+1):]
	url_keys = url_params_new.keys()
        url_keys.sort()
        cmd_str = ""
        for key in url_keys:
            cmd_str = cmd_str + "%s=%s&" % (key, url_params_new[key])

        #TEST
        cmd_str2 = cmd_str[:-1].lower()
        hashed = hmac.new(self.SECRET, cmd_str2, sha1)
        signature = binascii.b2a_base64(hashed.digest())[:-1]
        print "signature:\n%s" % signature
	return signature
         

         
    def execute(self, param=None):
        #Override
        #before execute, call paramCheck()
	if param != None:
	   self.updateParams(param)
	self.checkParams()
        cmd = self.createAPI()
        print "#"*40
	print "Requesting URL:\n%s" % cmd
        print "#"*40
        try:
            f = urllib2.urlopen(cmd)
            result = f.read()
        except:
            yn = raw_input("Call again(y/n)")
            if yn.lower() == "y":
                return self.execute(param)
            else:
                return False
            
	return result
	

    def parseJsonResult(self, result):
    	import json
    	args = json.loads(result)
    	return args

    def parseAsyncResult(self, dic):
    	keys = dic.keys()
	# key may be 1
	if len(keys) != 1:
	   print "Warning:%s" % dic
	key = keys[0]
	job = dic[key]
	id = job['id']
	jobid = job['jobid']
	return jobid
	    



from ucloudbiz.server.lib.api import UcloudAPI

class listTemplates(UcloudAPI):

    params = {	'templatefilter' : [True,None],
              	'account'       : [False,None],
      		'domain'    	: [False,None],
      		'hypervisor'    : [True,None],
      		'id'            : [False,None],
      		'keyword'       : [False,None],
      		'name'          : [False,None],
		'name' 		: [False,None],
		'page' 		: [False,None],
		'pagesize'	: [False,None],
		'zoneid'  	: [False,None],
		'isrecursive'	:[False,None],
		'listall'	:[False,None]
      }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class deleteTemplate(UcloudAPI):

    params = {	'id' : [True,None],
              	'zoneid'       : [False,None]
		  }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class updateTemplate(UcloudAPI):

    params = {	'id' 			: [True,None],
              	'bootable'      : [True,None],
				'displaytext'   : [True,None],
				'format'    	: [True,None],
				'name'          : [False,None],
				'ostypeid'      : [False,None],
				'passwordenabled' 	: [False,None],
				'sortkey'			:[False,None]
			  }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class createPortForwardingRule(UcloudAPI):

    params = {'ipaddressid'  : [True,None],
              'privateport'  : [True,None],
              'protocol'     : [True,None],
              'publicport'   : [True,None],
              'virtualmachineid'  : [True,None],
              'cidrlist'       : [False,None],
              'openfirewall'            : [False,None],
              'privateendport'                : [False,None],
              'publicendport'      	       : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class associateIpAddress(UcloudAPI):

    params = {'zoneid'        : [False,None],
              'usageplantype' : [False,None],
              'account'       : [False,None],
              'domainid'      : [False,None],
              'networkid'     : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)





from ucloudbiz.server.lib.api import UcloudAPI

class createFirewallRule(UcloudAPI):

    params = {'ipaddressid'  : [True,None],
              'protocol' : [True,None],
              'cidrlist'       : [False,None],
              'startport'            : [False,None],
              'endport'                : [False,None],
              'icmpcode'      	       : [False,None],
              'icmptype'               : [False,None],
              'type'            : [False,None],
              }
    
    def __init__(self):
        UcloudAPI.__init__(self)

    




from ucloudbiz.server.lib.api import UcloudAPI

class deployVirtualMachine(UcloudAPI):

    params = {'serviceofferingid' : [True,None],
              'templateid'        : [True,None],
              'diskofferingid'    : [True,None],
              'zoneid'            : [True,None],
              'name'              : [False,None],
              'networkids'        : [False,None],
              'userdata'          : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class queryAsyncJobResult(UcloudAPI):

    params = {'jobid' : [True,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    def getResult(self, result):
        # @param result : result dictionary
        # key : queryasyncjobresultresponse
        rvalue = result['queryasyncjobresultresponse']
        return (int(rvalue['jobstatus']), rvalue)




from ucloudbiz.server.lib.api import UcloudAPI

class listVirtualMachines(UcloudAPI):

    params = {
        'account' : [False,None],
        'domainid' : [False,None],
        'forvirtualnetwork' : [False, None],
        'groupid' : [False, None],
        'hostid' : [False, None],
        'hypervisor' : [False, None],
        'id' : [False, None],
        'isrecursive' : [False, None],
        'keyword' : [False, None],
        'name' : [False, None],
        'networkid' : [False, None],
        'page' : [False, None],
        'pagesize' : [False, None],
        'podid' : [False, None],
        'state' : [False, None],
        'storageid' : [False, None],
        'zoneid' : [False, None],
        'details' : [False, None],
        'listall' : [False, None],
        }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class listAccounts(UcloudAPI):

    params = {'accounttype'       : [False,None],
              'domainid'       : [False,None],
              'id'             : [False,None],
              'iscleanuprequired'     : [False,None],
              'isrecursive'     : [False,None],
              'keyword'     : [False,None],
              'name'     : [False,None],
              'page'     : [False,None],
              'pagesize'     : [False,None],
              'state'     : [False,None],
              'listall'     : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)





import os
from setuptools import setup, Extension
from glob import glob

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "ucloudbiz",
    version = "0.2.2",
    author = "Choonho Son",
    author_email = "choonho.son@kt.com",
    description = ("ucloudbiz openapi python library."),
    license = "BSD",
    keywords = "ucloud ucloudbiz config library",
    url = "http://ucloud.googlecode.com",
    packages=['ucloudbiz', 'ucloudbiz.server', 'ucloudbiz.server.lib', 'ucloudbiz.server.vm', 'ucloudbiz.server.address', 'ucloudbiz.server.portforwarding','ucloudbiz.server.template'],
    long_description=read('README'),
    classifiers=[
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    zip_safe=True,
)



from ucloudbiz.server.lib.api import UcloudAPI

class createTemplate(UcloudAPI):

    params = {	'displaytext' : [True,None],
              	'name'       : [True,None],
      		'ostypeid'    	: [True,None],
      		'bit'    : [True,None],
      		'details'            : [False,None],
      		'isfeatured'       : [False,None],
      		'ispublic'          : [False,None],
		'passwordenabled' 		: [False,None],
		'requireshvm' 		: [False,None],
		'snapshotid'	: [False,None],
		'templatetag'  	: [False,None],
		'url'	:[False,None],
		'virtualmachineid'	:[False,None],
		'volumeid'	:[False,None]
      }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    




from ucloudbiz.server.lib.api import UcloudAPI

class listPublicIpAddresses(UcloudAPI):

    params = {'account'       : [True,None],
              'allocatedonly' : [False,None],
              'domainid'       : [False,None],
              'forvirtualnetwork'      : [False,None],
              'id'     : [False,None],
              'ipaddress'     : [False,None],
              'keyword'     : [False,None],
              'page'     : [False,None],
              'pagesize'     : [False,None],
              'vlanid'     : [False,None],
              'associatednetworkid'     : [False,None],
              'isrecursive'     : [False,None],
              'issourcenat'     : [False,None],
              'isstaticat'     : [False,None],
              'listall'     : [False,None],
              'physicalnetworkid'     : [False,None],
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)





from ucloudbiz.server.lib.api import UcloudAPI

class listAvailableProductTypes(UcloudAPI):

    params = {
				'zoneid':[False,None]
              }
    
    def __init__(self, url=None, apikey=None, secret=None):
        UcloudAPI.__init__(self, url, apikey, secret)

    

